# -*- coding: utf-8 -*-
#
# Copyright © 2009-2010 CEA
# Pierre Raybaut
# Licensed under the terms of the CECILL License
# (see guidata/__init__.py for details)

"""
Scan this module to include new translatable strings
must be run from toplevel directory!
"""
from guidata.gettext_helpers import do_rescan

if __name__ == "__main__":
    do_rescan("guidata")
